﻿Imports System.ComponentModel
Imports System.Drawing
Imports System.Windows.Forms

Public Class WindowsFormNativeWindowListener
    Inherits NativeWindow

#Region "Delegates and Event Definitions"

    Public Event WindowMinimizing As CancelEventHandler
    Public Event WindowMaximizing As CancelEventHandler
    Public Event WindowRestoring As CancelEventHandler

    Public Delegate Sub WindowStateChangedEventHandler(ByVal sender As Object, ByVal e As WindowStateChangedEventArgs)

    Public Event WindowMinimized As WindowStateChangedEventHandler
    Public Event WindowMaximized As WindowStateChangedEventHandler
    Public Event WindowRestored As WindowStateChangedEventHandler

#End Region

#Region "Fields and Constants"

    Private WithEvents _Timer As Timer

    Private WithEvents _ParentForm As Form = Nothing

    Private _NewWindowState As FormWindowState
    Private _OldWindowState As FormWindowState

    Private _CancelEventArgs As CancelEventArgs = Nothing

    Private _OldBounds As Rectangle

    Private _RaiseEvent As Boolean
    Private _OldVisible As Boolean

    Private Const _TIMER_DELAY_IN_MILLI_SECONDS As Integer = 50

#Region "Win32 API Constants"

    Private Const WM_SYSCOMMAND As Int32 = &H112

    Private Const SC_CLOSE As Int32 = &HF060
    Private Const SC_MAXIMIZE As Int32 = &HF030
    Private Const SC_MINIMIZE As Int32 = &HF020
    Private Const SC_RESTORE As Int32 = &HF120

#End Region

#End Region

#Region "Constructors"

    Public Sub New(ByVal parentForm As Form)
        MyBase.New()
        _ParentForm = parentForm
        Me.AssignHandle(parentForm.Handle)
    End Sub 'New

#End Region

#Region "Events"

    Private Sub ParentForm_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles _ParentForm.FormClosed
        Me.ReleaseHandle()
    End Sub 'ParentForm_FormClosed

    Private Sub _Timer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles _Timer.Tick
        _Timer.Stop()
        _Timer.Dispose()
        _Timer = Nothing

        Dim windowStateChangedEventArgs As New WindowStateChangedEventArgs(_OldBounds, _OldVisible, _OldWindowState, _ParentForm)
        Select Case _NewWindowState
            Case FormWindowState.Maximized
                RaiseEvent WindowMaximized(_ParentForm, windowStateChangedEventArgs)

            Case FormWindowState.Minimized
                RaiseEvent WindowMinimized(_ParentForm, windowStateChangedEventArgs)

            Case FormWindowState.Normal
                RaiseEvent WindowRestored(_ParentForm, windowStateChangedEventArgs)
        End Select
    End Sub 'Timer_Tick

#End Region

#Region "Methods"

#Region "Overridden"

    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = WM_SYSCOMMAND Then
            Select Case m.WParam.ToInt32()
                Case SC_MAXIMIZE
                    ' Allow form window state change to be cancelled.
                    _CancelEventArgs = New CancelEventArgs()
                    OnWindowMaximizing(_CancelEventArgs)
                    If _CancelEventArgs.Cancel Then
                        Return
                    Else
                        _RaiseEvent = True
                        _NewWindowState = FormWindowState.Maximized
                        ' State change does not occur until MyBase.WndProc is called below.
                        _OldBounds = _ParentForm.Bounds : _OldVisible = _ParentForm.Visible : _OldWindowState = _ParentForm.WindowState
                    End If

                Case SC_MINIMIZE
                    ' Allow form window state change to be cancelled.
                    _CancelEventArgs = New CancelEventArgs()
                    OnWindowMinimizing(_CancelEventArgs)
                    If _CancelEventArgs.Cancel Then
                        Return
                    Else
                        _RaiseEvent = True
                        _NewWindowState = FormWindowState.Minimized
                        ' State change does not occur until MyBase.WndProc is called below.
                        _OldBounds = _ParentForm.Bounds : _OldVisible = _ParentForm.Visible : _OldWindowState = _ParentForm.WindowState
                    End If

                Case SC_RESTORE
                    ' Allow form window state change to be cancelled.
                    _CancelEventArgs = New CancelEventArgs()
                    OnWindowRestoring(_CancelEventArgs)
                    If _CancelEventArgs.Cancel Then
                        Return
                    Else
                        _RaiseEvent = True
                        _NewWindowState = FormWindowState.Normal
                        ' State change does not occur until MyBase.WndProc is called below.
                        _OldBounds = _ParentForm.Bounds : _OldVisible = _ParentForm.Visible : _OldWindowState = _ParentForm.WindowState
                    End If
            End Select
        End If

        MyBase.WndProc(m)

        ' Raise window state changed events - Changes have been applied after call to MyBase.WndProc.
        If _RaiseEvent Then
            _RaiseEvent = False
            _Timer = New Timer()
            _Timer.Interval = _TIMER_DELAY_IN_MILLI_SECONDS
            _Timer.Start()
        End If
    End Sub 'WndProc

#End Region

#Region "Protected"

    Protected Sub OnWindowMaximizing(ByVal e As CancelEventArgs)
        RaiseEvent WindowMaximizing(_ParentForm, e)
    End Sub 'OnWindowMaximizing

    Protected Sub OnWindowMaximized(ByVal e As WindowStateChangedEventArgs)
        RaiseEvent WindowMaximized(_ParentForm, e)
    End Sub 'OnWindowMaximized

    Protected Sub OnWindowMinimizing(ByVal e As CancelEventArgs)
        RaiseEvent WindowMinimizing(_ParentForm, e)
    End Sub 'OnWindowMinimizing

    Protected Sub OnWindowMinimized(ByVal e As WindowStateChangedEventArgs)
        RaiseEvent WindowMinimized(_ParentForm, e)
    End Sub 'OnWindowMinimized

    Protected Sub OnWindowRestoring(ByVal e As CancelEventArgs)
        RaiseEvent WindowRestoring(_ParentForm, e)
    End Sub 'OnWindowRestoring

    Protected Sub OnWindowRestored(ByVal e As WindowStateChangedEventArgs)
        RaiseEvent WindowRestored(_ParentForm, e)
    End Sub 'OnWindowRestored

#End Region

#End Region

End Class 'WindowsFormNativeWindowListener
